<?php

/**
 * Return the version string.
 *
 * @author Eric Lubow <elubow@simplereach.com>
 * @param None
 * @return String Version
 */
function sr_plugin_version()
{
    return SRSLIDE_PLUGIN_VERSION;
}

/**
 * Convert the $_POST variable into a style hash
 *
 * @author Eric Lubow <elubow@simplereach.com>
 * @param Object $POST Post object being shown on the page
 * @return Array $style An associative array of styles for The Slide
 */
function create_style_hash($POST)
{
    $style = array();

    // Iterate over our $_POST var and rip out the styles to put into a hash
    foreach ($POST as $name => $value) {
        if (!preg_match('/^srslide_style_(.+?)_(.+?)$/',$name,$style_name)) { continue; }
        $style[$style_name[1]][$style_name[2]] = $value;
    }

    return clean_style_options($style);
}

/**
 * Remove the extra style options prior to save/send to SimpleReach
 *
 * @author Eric Lubow <elubow@simplereach.com>
 * @param Array $style The associative array of styles for The Slide
 * @return Array $style The clean associative array of styles for The Slide
 */
function clean_style_options($style)
{
    // Remove the link_color_text and just have link_color
    if ($style['link']['color'] == 'other') {
        $style['link']['color'] = $style['link']['color_text'];
    }
    unset($style['link']['color_text']);

    return $style;
}

/**
 * Validate css hex color:
 * Must begin with '#'
 * May only contain [0-9a-f]
 * Must contain either 3 or 6 chars
 *
 * @author Malaney J. Hill <malaney@gmail.com>
 * @param String $value string
 * @return Boolean true if valid / false if invalid
 */
function validate_hex_color($val)
{
    return preg_match('/^#[a-f0-9]{3}$/i', $val) || preg_match('/^#[a-f0-9]{6}$/i', $val);
}


/**
 * Put together some debug info to send to SimpleReach when support
 * is requested.
 *
 * @author Eric Lubow <elubow@simplereach.com>
 * @param None
 * @return String Debug info
 */
function create_debug_info()
{
    $attrs                       = array();
    $attrs['pid']                = get_option('srslide_pid');
    $attrs['email']              = get_option('srslide_sr_login');
    $attrs['website']            = get_option('siteurl');
    $attrs['account']            = get_option('blogname');
    $attrs['user_api_key']       = get_option('srslide_api_key');
    $attrs['wp_version']         = get_bloginfo('version');
    $attrs['plugin_version']     = sr_plugin_version();
    $attrs['show_slide_logo']    = get_option('srslide_show_logo');
    $attrs['header_text']        = get_option('srslide_header_text');
    $attrs['use_wp_pages']       = get_option('srslide_use_pages');
    $attrs['styles']             = serialize(get_option('srslide_styles'));
    $attrs['account']            = get_option('blogname');
    $attrs['php_version']        = phpversion();
    $attrs['curl_version']       = _srslide_get_curl_version();
    $attrs['curl_ssl']           = _srslide_get_curl_ssl();
    $attrs['curl_https_enabled'] = _srslide_get_curl_ssl_enabled();
    $attrs['wp_plugins']         = _srslide_get_wp_plugins();
    if ($theme_name = get_current_theme()) {
	$attrs['theme']          = $theme_name;
	$theme_data = get_theme_data(get_stylesheet_directory().'/style.css' );
	if (is_array($theme_data)) {
		foreach ($theme_data as $k=>$v) {
			$attrs['theme_'.$k] = $v;
		}
	}
    }
    $str = '';
    foreach ($attrs as $attr => $value) {
        $str .= $attr .': ' . $value . "\n";
    }
    return $str;
}

function get_channels($post)
{
    $cat_as_chan = get_option('srslide_cat_as_chan');
    if (!$cat_as_chan) { return ''; }

    $post_categories = wp_get_post_categories($post->ID);
    $myCats = array();
    foreach ($post_categories as $c) {
        $cat = get_category($c);
        $myCats[] = $cat->slug;
    }
    $cats = join(',', $myCats);

    return addslashes($cats);
}

function _srslide_get_wp_plugins()
{
    $plugins = get_plugins();
    if ($plugins) {
        foreach ($plugins as $plugin_file => $plugin_data) {
            if (is_plugin_active($plugin_file)) {
                $results[] = $plugin_data['Title'] .'-' . $plugin_data['Version'];
            }
        }
    }
    return join(',', $results);
}

function _srslide_get_curl_version()
{
    if (extension_loaded('curl')) {
        $cv = curl_version();
        return $cv['version'];
    } else {
        return 'NONE';
    }
}

function _srslide_get_curl_ssl()
{
    if (extension_loaded('curl')) {
        $cv = curl_version();
        if (!empty($cv['ssl_version'])) {
            return $cv['ssl_version'];
        } else {
            return 'NONE';
        }
    } else {
        return 'NONE';
    }
}

function _srslide_get_curl_ssl_enabled()
{
    if (extension_loaded('curl')) {
        $cv = curl_version();
        if (!empty($cv['protocols'])) {
            $protocols = $cv['protocols'];
            return (in_array('https', $protocols)) ? 'YES' : 'NO';
        } else {
            return 'NO';
        }
    } else {
        return 'NO';
    }
}

/**
 * Return the SimpleReach title for the post.
 *
 * @author Eric Lubow <elubow@simplereach.com>
 * @param Object $post Post object being shown on the page
 * @return String Title of the post with slashes escaped
 */
function get_sr_post_title($post)
{
    $title = get_post_meta($post->ID, '_srslide_title', true);
    if (empty($title)) {
        $title = $post->post_title;
    }
    return addslashes($title);
}

function get_sr_post_article_icon($post)
{
    $icon = get_post_meta($post->ID, '_srslide_article_icon', true);
    return $icon;
}


/**
 * Return the SimpleReach tags for the post.
 *
 * @author Eric Lubow <elubow@simplereach.com>
 * @param Object $post Post object being shown on the page
 * @return String Tags of the post with slashes escaped
 */
function get_sr_post_tags($post)
{
    $tags = get_post_meta($post->ID, '_srslide_tags', true);
    if (empty($tags)) {
        $wptags = wp_get_post_tags($post->ID);
        $myTags = array();
        foreach ($wptags as $tag) {
            $myTags[] = (is_object($tag)) ? $tag->name : $tag;
        }
	$tags = join(',', $myTags);
    }
    return addslashes($tags);
}


/**
 * Set some defaults for the plugin
 *
 * @author Eric Lubow <elubow@simplereach.com>
 * @param Object $post_data The result of the $_POST data from the registration
 * @return None
 */
function set_srslide_defaults($post_data)
{
    update_option('srslide_sr_login', $post_data['srslide_sr_login']);
    update_option('srslide_sr_pass', $post_data['srslide_sr_pass']);
    update_option('srslide_registered', true);
    update_option('srslide_show_logo', ($post_data['srslide_show_logo'] == 'on'));
    update_option('srslide_header_text', 'RECOMMENDED FOR YOU');
    update_option('srslide_use_pages', false);
    update_option('srslide_cat_as_chan', false);

    $style_defaults = array(
        'element' => array(
            'bottom' => '0px'
        ),
        'link' => array(
            'color' => 'FE3F10'
        )
    );
    update_option('srslide_styles',$style_defaults);
}


/**
 * Update The Slide styles on the SimpleReach site
 *
 * @author Eric Lubow <elubow@simplereach.com>
 * @param Array $styles Style to be converted to JSON
 * @return String OK or error string
 */
function update_slide_styles()
{
    // Pull all the necessary options
    // Setup our POST parameters
    $params = array();
    $params['ts'] = (int)gmdate('U');
    $params['pid'] = get_option('srslide_pid');

    $srslide_api_key = get_option('srslide_api_key');
    // Handle if the user's API key isn't saved in WP
    if (strlen($srslide_api_key) == 32) {
	$params['api_key'] = get_option('srslide_api_key');
    } else {
	$params['login'] = get_option('srslide_sr_login');
	$params['pass'] = get_option('srslide_sr_pass');
    }

    $params['style'] = get_option('srslide_styles');
    if (!validate_hex_color($params['style']['link']['color'])) {
	$errorMessage = __("Please specify a valid link color which starts with a '#' and has 6 of the following characters [A-F], [0-9]", 'srslide');
	return $errorMessage;
    }
    $encoded_params = http_build_query($params);

    $url = "https://www.simplereach.com/publisher/generate_slide_css";
    $response = srslide_post_url($url, $encoded_params);

    // Decode the JSON
    $resp = json_decode($response);
    if ($resp === NULL) {
	$errorMessage = __("There was an http error contacting SimpleReach. Please try saving your options again.", 'srslide');
	return $errorMessage;
    }

    if ($resp->{'success'}) {
        if ($resp->{'api_key'} && !api_key_exists()) {
            update_option('srslide_api_key',$resp->{'api_key'});
        }
        return "OK";
    } else {
	$errorResponse = (!empty($resp->{'error'})) ? strtolower($resp->{'error'}) : '';
	$errorFooter =  sprintf(__("Please try again or contact <a href='mailto:%s'>SimpleReach support</a> if you believe you have received this message in error.", 'srslide'), SRSLIDE_PLUGIN_SUPPORT_EMAIL);
	switch ($errorResponse) {
		case 'incorrect timestamp':
			$errorMessage = __("There was an error while you were attempting to register.", 'srslide') . $errorFooter;
		break;
		case 'invalid pid':
            		$errorMessage = __("The PID (publisher ID) that you are attempting to sign in with does not exist.", 'srslide') . $errorFooter;
		break;
		case 'invalid user':
			$errorMessage = __("The user name that you are attempting to sign in with does not exist.", 'srslide') . $errorFooter;
		break;
		case 'invalid password':
			$errorMessage = __("The user name that you are attempting to register with already exists, but the password you entered was incorrect.", 'srslide') . $errorFooter;
		break;
		case 'invalid api key':
			$errorMessage = __("The API key that you are using does not have access to this account or is invalid.", 'srslide') . $errorFooter;
		break;
		case 'invalid user on account':
			$errorMessage = __("The user name that you are registered with does not have access to this account.", 'srslide') . $errorFooter;
		break;
		default:
			$errorMessage = __("There was an error contacting SimpleReach. Please try saving your options again.", 'srslide');
		break;
	}
	return $errorMessage;
    }
}


/**
 * Check to see if the API key exists
 *
 * @author Eric Lubow <elubow@simplereach.com>
 * @param None
 * @return Boolean
 */
function api_key_exists()
{
	$existing_api_key = get_option('srslide_api_key');
	return strlen($existing_api_key) == 32;
}

/**
 * srslide_post_url
 *
 * @param mixed $url
 * @param array $data
 * @access public
 * @return void
 */
function srslide_post_url($url, $data=array())
{
    try {
        if (extension_loaded('curl')) {
            return _srslideCurlPostUrl($url, $data);
        } else {
            return _srslideStreamPostUrl($url, $data);
        }
    } catch (Exception $e) {
        $url = preg_replace('/https/', 'http', $url);

        if (extension_loaded('curl')) {
            return _srslideCurlPostUrl($url, $data);
        } else {
            return _srslideStreamPostUrl($url, $data);
        }
    }
}

function _srslideCurlPostUrl($url, $data=array()) {
    $curl = curl_init();
    curl_setopt($curl, CURLOPT_URL, $url);
    curl_setopt($curl, CURLOPT_POST, true);
    curl_setopt($curl, CURLOPT_TIMEOUT, 30);
    curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
    $cv = curl_version();
    // Is SSL Supported?
    if ($cv['features'] && constant('CURL_VERSION_SSL')) {
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
    }

    $result = curl_exec($curl);
    if ($result === false) {
        $error = curl_error($curl);
        curl_close($curl);
        throw new Exception($error);
    }
    curl_close($curl);
    return $result;
}

function _srslideStreamPostUrl($url, $data=array())
{
    if (!extension_loaded('openssl')) {
        $url = preg_replace('/https/', 'http', $url);
    }

    $params = array('http' => array(
                'method' => 'POST',
                'content' => $data
                ));
    if ($optional_headers !== null) {
        $params['http']['header'] = $optional_headers;
    }
    $ctx = stream_context_create($params);
    $fp = @fopen($url, 'rb', false, $ctx);
    if (!$fp) {
        throw new Exception("Problem with $url, $php_errormsg");
    }
    $response = @stream_get_contents($fp);
    if ($response === false) {
        throw new Exception("Problem reading data from $url, $php_errormsg");
    }
    return $response;
}

function srslide_validate_login($login) {
    $emailValidatorRegex = '/^[-a-z0-9~!$%^&*_=+}{\'?]+(\.[-a-z0-9~!$%^&*_=+}{\'?]+)*@([a-z0-9_][-a-z0-9_]*(\.[-a-z0-9_]+)*\.(aero|arpa|biz|com|coop|edu|gov|info|int|mil|museum|name|net|org|pro|travel|mobi|[a-z][a-z])|([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}))(:[0-9]{1,5})?$/i';
    return preg_match($emailValidatorRegex, $login);
}

function srslide_validate_password($password) {
    return preg_match('/^[a-z0-9@#\$%^\+\&\_\-\!\;\,]{6,99}$/i', $password);
}
?>
